package ru.dbotthepony.kstarbound

import com.google.common.collect.ImmutableMap
import com.google.gson.Gson
import com.google.gson.GsonBuilder
import com.google.gson.TypeAdapter
import com.google.gson.TypeAdapterFactory
import com.google.gson.reflect.TypeToken
import com.google.gson.stream.JsonReader
import java.util.Arrays
import java.util.stream.Stream
import kotlin.reflect.KProperty
import kotlin.reflect.full.createType

inline fun <reified T> GsonBuilder.registerTypeAdapter(adapter: TypeAdapter<T>): GsonBuilder {
	return registerTypeAdapter(T::class.java, adapter)
}

inline fun <reified T> GsonBuilder.registerTypeAdapter(noinline factory: (Gson) -> TypeAdapter<T>): GsonBuilder {
	val token = TypeToken.get(T::class.java)

	return registerTypeAdapterFactory(object : TypeAdapterFactory {
		override fun <T : Any?> create(gson: Gson, type: TypeToken<T>): TypeAdapter<T>? {
			if (type == token) {
				return factory(gson) as TypeAdapter<T>
			}

			return null
		}
	})
}

fun <T> Array<T>.stream(): Stream<T> = Arrays.stream(this)

operator fun <T> ThreadLocal<T>.getValue(thisRef: Any, property: KProperty<*>): T? {
	return get()
}

operator fun <T> ThreadLocal<T>.setValue(thisRef: Any, property: KProperty<*>, value: T?) {
	set(value)
}

operator fun <K, V> ImmutableMap.Builder<K, V>.set(key: K, value: V): ImmutableMap.Builder<K, V> = put(key, value)

fun String.sintern(): String = Starbound.strings.intern(this)

inline fun <reified T> Gson.fromJson(reader: JsonReader): T? = fromJson<T>(reader, T::class.java)

