package ru.dbotthepony.kstarbound.client.render

import org.lwjgl.opengl.GL45.*
import ru.dbotthepony.kbox2d.api.IDebugDraw
import ru.dbotthepony.kbox2d.api.Transform
import ru.dbotthepony.kstarbound.PIXELS_IN_STARBOUND_UNIT
import ru.dbotthepony.kstarbound.client.StarboundClient
import ru.dbotthepony.kstarbound.client.gl.vertex.GeometryType
import ru.dbotthepony.kstarbound.client.gl.vertex.StreamVertexBuilder
import ru.dbotthepony.kstarbound.client.gl.vertex.VertexAttributes
import ru.dbotthepony.kvector.arrays.Matrix3f
import ru.dbotthepony.kvector.vector.RGBAColor
import ru.dbotthepony.kvector.vector.Vector2d
import kotlin.math.cos
import kotlin.math.sin

class Box2DRenderer : IDebugDraw {
	val state = StarboundClient.current()
	private val identity = Matrix3f.identity()

	override var drawShapes: Boolean = false
	override var drawJoints: Boolean = false
	override var drawAABB: Boolean = false
	override var drawTreeAABB: Boolean = false
	override var drawPairs: Boolean = false
	override var drawCenterOfMess: Boolean = false

	private val builder = StreamVertexBuilder(VertexAttributes.POSITION)

	override fun drawPolygon(vertices: List<Vector2d>, color: RGBAColor) {
		require(vertices.size > 1) { "Vertex list had only ${vertices.size} namings in it" }

		builder.builder.begin()

		for (i in vertices.indices) {
			val current = vertices[i]
			val next = vertices[(i + 1) % vertices.size]
			builder.builder.vertex(current.x.toFloat(), current.y.toFloat())
			builder.builder.vertex(next.x.toFloat(), next.y.toFloat())
		}

		builder.upload()

		state.programs.position.use()
		state.programs.position.colorMultiplier = color
		state.programs.position.modelMatrix = state.stack.last()

		builder.draw(GL_LINES)
	}

	private fun drawSolid(vertices: List<Vector2d>, color: RGBAColor) {
		require(vertices.size >= 3) { "Vertex list had only ${vertices.size} namings in it" }

		builder.builder.begin(GeometryType.TRIANGLES)

		val zero = vertices[0]

		for (i in 1 until vertices.size) {
			val current = vertices[i]
			val next = vertices[(i + 1) % vertices.size]
			builder.builder.vertex(zero.x.toFloat(), zero.y.toFloat())
			builder.builder.vertex(current.x.toFloat(), current.y.toFloat())
			builder.builder.vertex(next.x.toFloat(), next.y.toFloat())
		}

		builder.upload()

		state.programs.position.use()
		state.programs.position.colorMultiplier = color
		state.programs.position.modelMatrix = state.stack.last()

		builder.draw(GL_TRIANGLES)
	}

	override fun drawSolidPolygon(vertices: List<Vector2d>, color: RGBAColor) {
		drawSolid(vertices, color.copy(alpha = 0.5f))
		drawPolygon(vertices, color)
	}

	override fun drawCircle(center: Vector2d, radius: Double, color: RGBAColor) {
		val vertexList = ArrayList<Vector2d>()

		for (i in 0 until 360 step 15) {
			val rad = Math.toRadians(i.toDouble())
			val c = cos(rad)
			val s = sin(rad)

			vertexList.add(Vector2d(
				center.x + c * radius,
				center.y + s * radius
			))
		}

		drawPolygon(vertexList, color)
	}

	override fun drawSolidCircle(center: Vector2d, radius: Double, axis: Vector2d, color: RGBAColor) {
		val vertexList = ArrayList<Vector2d>()

		for (i in 0 until 360 step 15) {
			val rad = Math.toRadians(i.toDouble())
			val c = cos(rad)
			val s = sin(rad)

			vertexList.add(Vector2d(
				center.x + c * radius,
				center.y + s * radius
			))
		}

		drawSolidPolygon(vertexList, color.copy(alpha = 0.5f))
		drawPolygon(vertexList, color)
		drawPolygon(listOf(center, center + axis * radius), color)
	}

	override fun drawSegment(p1: Vector2d, p2: Vector2d, color: RGBAColor) {
		drawPolygon(listOf(p1, p2), color)
	}

	override fun drawTransform(xf: Transform) {
		TODO("Not yet implemented")
	}

	override fun drawPoint(p: Vector2d, size: Double, color: RGBAColor) {
		drawSolid(listOf(
			Vector2d(x = p.x - size / (PIXELS_IN_STARBOUND_UNIT * 2.0), y = p.y - size / (PIXELS_IN_STARBOUND_UNIT * 2.0)),
			Vector2d(x = p.x + size / (PIXELS_IN_STARBOUND_UNIT * 2.0), y = p.y - size / (PIXELS_IN_STARBOUND_UNIT * 2.0)),
			Vector2d(x = p.x + size / (PIXELS_IN_STARBOUND_UNIT * 2.0), y = p.y + size / (PIXELS_IN_STARBOUND_UNIT * 2.0)),
			Vector2d(x = p.x - size / (PIXELS_IN_STARBOUND_UNIT * 2.0), y = p.y + size / (PIXELS_IN_STARBOUND_UNIT * 2.0)),
		), color)
	}
}
