package ru.dbotthepony.kstarbound.defs.animation

import com.google.common.collect.ImmutableList
import com.google.common.collect.ImmutableMap
import ru.dbotthepony.kstarbound.defs.image.SpriteReference
import ru.dbotthepony.kstarbound.defs.particle.ParticleEmitter
import ru.dbotthepony.kstarbound.io.json.builder.JsonFactory
import ru.dbotthepony.kstarbound.util.Either
import ru.dbotthepony.kvector.vector.Vector2d

@JsonFactory
data class AnimationDefinition(
	val frames: SpriteReference? = null,
	val variants: Int? = null,
	val frameNumber: Int? = null,
	val animationCycle: Double? = null,
	val offset: Vector2d? = null,

	val animatedParts: AnimatedParts? = null,

	val sounds: ImmutableMap<String, Either<ImmutableList<String>, CustomSound>> = ImmutableMap.of(),
	val transformationGroups: ImmutableMap<String, TransformConfig> = ImmutableMap.of(),
	val particleEmitters: ImmutableMap<String, ParticleEmitter> = ImmutableMap.of(),
) {
	@JsonFactory
	data class CustomSound(
		val pool: ImmutableList<String>,
	)

	@JsonFactory
	data class TransformConfig(
		val interpolated: Boolean? = null
	)

	@JsonFactory
	data class AnimatedParts(
		val stateTypes: ImmutableMap<String, StateType> = ImmutableMap.of(),
		val parts: ImmutableMap<String, Part> = ImmutableMap.of(),
	) {
		@JsonFactory
		data class StateType(
			val default: String,
			val priority: Int = 0,
			val states: ImmutableMap<String, State> = ImmutableMap.of(),
		) {
			@JsonFactory
			data class State(
				val frames: Int = 0,
				val cycle: Double = 0.0,
				val mode: Mode? = null,
				val transition: TransitionType? = null,
				val properties: Properties = Properties()
			) {
				enum class TransitionType {
					NONE
				}

				enum class Mode {
					TRANSITION,
					LOOP,
				}

				@JsonFactory
				data class Properties(
					val immediateSound: String? = null
				)
			}
		}

		@JsonFactory
		data class Part(
			val properties: Properties = Properties.EMPTY,
			val partStates: ImmutableMap<String, ImmutableMap<String, State>> = ImmutableMap.of(),
		) {
			@JsonFactory
			data class Properties(
				val fullbright: Boolean? = null,
				val centered: Boolean? = null,
				val transformationGroups: ImmutableList<String>? = null,
				val offset: Vector2d? = null,
				val image: SpriteReference? = null
			) {
				companion object {
					val EMPTY = Properties()
				}
			}

			@JsonFactory
			data class State(
				val properties: Properties = Properties.EMPTY
			)
		}
	}
}
