package ru.dbotthepony.kstarbound.world.api

import ru.dbotthepony.kstarbound.RegistryObject
import ru.dbotthepony.kstarbound.defs.tile.LiquidDefinition
import ru.dbotthepony.kstarbound.defs.tile.MaterialModifier
import ru.dbotthepony.kstarbound.defs.tile.TileDefinition
import ru.dbotthepony.kvector.api.IStruct2i
import java.io.DataInputStream

interface IChunkCell : IStruct2i {
	/**
	 * absolute (in world)
	 */
	val x: Int

	/**
	 * absolute (in world)
	 */
	val y: Int

	override fun component1(): Int {
		return x
	}

	override fun component2(): Int {
		return y
	}

	val foreground: ITileState
	val background: ITileState
	val liquid: ILiquidState

	var dungeonId: Int
	var biome: Int
	var envBiome: Int
	var isIndestructible: Boolean

	fun read(stream: DataInputStream) {
		foreground.read(stream)
		background.read(stream)
		liquid.read(stream)

		stream.skipNBytes(1) // collisionMap

		dungeonId = stream.readUnsignedShort()
		biome = stream.readUnsignedByte()
		envBiome = stream.readUnsignedByte()
		isIndestructible = stream.readBoolean()

		stream.skipNBytes(1) // unknown
	}

	companion object {
		fun skip(stream: DataInputStream) {
			ITileState.skip(stream)
			ITileState.skip(stream)
			ILiquidState.skip(stream)
			stream.skipNBytes(1 + 2 + 1 + 1 + 1 + 1)
		}
	}
}